// RS232Interface.cpp: implementation of the CRS232Interface class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CommTest.h"
#include "RS232Interface.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS232Interface::CRS232Interface
//
// Description:   Opens the communications channel with the I/O module.  
//                Only one version of this object can be instantiated
//                throughout the system, so there is a critical section lock
//                at the beginning of the constructor, and an unlock in the
//                destructor
/////////////////////////////////////////////////////////////////////////////
CRS232Interface::CRS232Interface(CString ComPort, int BaudRate)
{
   // Lock the serial object so only one can be used at a time (this is now a multiple-thread
   // safe object)
   m_CommCritSect.Lock();

   DCB dcb;
   BOOL fSuccess;

   if (TRUE) { // Will eventually have conditions for whether or not we do this, I think...
      // Must be OPEN_EXISTING... strPort should be "COM1", "COM2", etc...
      CString strPort = ComPort;
      m_hCom = CreateFile(strPort.GetBuffer(strPort.GetLength()+1), GENERIC_READ | GENERIC_WRITE, 0,
                          NULL, OPEN_EXISTING, 0, NULL);

      if (m_hCom == INVALID_HANDLE_VALUE) {
          // Handle error
         TRACE("CRS232Interface::CRS232Interface -- Invalid Comm Handle\n");
         return;
      }

      fSuccess = GetCommState(m_hCom, &dcb);

      if (!fSuccess) {
          // Handle the error
         TRACE("CRS232Interface::CRS232Interface -- GetCommState() error\n");
      }

      // Fill in the DCB: baud=whatever is set, 8 data bits, no parity, 1 stop bit
      dcb.BaudRate = BaudRate;
      dcb.ByteSize = 8;
      dcb.Parity   = NOPARITY;
      dcb.StopBits = ONESTOPBIT;

      // I'm sure stuff could be different here... but this works so don't touch it :P
      dcb.fOutxCtsFlow = FALSE;
      dcb.fOutxDsrFlow = FALSE;
      dcb.fDtrControl = DTR_CONTROL_ENABLE;
      dcb.fRtsControl = RTS_CONTROL_DISABLE;
      dcb.fDsrSensitivity = FALSE;
      dcb.fTXContinueOnXoff = TRUE;
      dcb.fOutX = FALSE; // **IMPORTANT**
      dcb.fInX = FALSE; // **IMPORTANT**
      dcb.fAbortOnError = FALSE;
      dcb.EvtChar = '\0';

      fSuccess = SetCommState(m_hCom, &dcb);

      if (!fSuccess) {
          // Handle the error.
         TRACE("CRS232Interface::CRS232Interface -- SetCommState() error\n");
      }

      // Important... set our timeout values
      COMMTIMEOUTS cto;

      cto.ReadIntervalTimeout         = 20 * (19200/BaudRate);
      cto.ReadTotalTimeoutMultiplier  = 0;
      cto.ReadTotalTimeoutConstant    = 10000;
      cto.WriteTotalTimeoutMultiplier = 0;
      cto.WriteTotalTimeoutConstant   = 750;
      fSuccess = SetCommTimeouts(m_hCom, &cto);

      if (!fSuccess) {
         // Handle the error.
         TRACE("CRS232Interface::CRS232Interface -- SetCommTimeouts() error\n");
      }
   }
   else {
      // We're in demo mode... make the handle invalid so that none of the 
      // functions get used.
      m_hCom = INVALID_HANDLE_VALUE;
   }

   m_iInterfaceType = MODE_RS232;
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS232Interface::~CRS232Interface
//
// Description:   Destructor... closes the handle to the com device and unlocks
//                the critical section.
/////////////////////////////////////////////////////////////////////////////
CRS232Interface::~CRS232Interface()
{
   // Close the file handle
   if (IsValidComm()) {
      CloseHandle(m_hCom);
   }

   // Make sure we unlock the serial object!
   m_CommCritSect.Unlock();
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS232Interface::SendCommand
//
// Description:   Sends and validates a command string to the instrument
/////////////////////////////////////////////////////////////////////////////
BOOL CRS232Interface::SendCommand(CString strCmd)
{
   ULONG numout;

   if (IsValidComm()) {
      // Write the buffer out to the com port
      //WriteFile(m_hCom, strCmd.GetBuffer(0), strCmd.GetLength(), &numout, NULL);

      int retval = WriteFile(m_hCom, strCmd.GetBuffer(0), strCmd.GetLength(), &numout, NULL);


      if (retval && numout == (ULONG)strCmd.GetLength()) {
         // Ok, we sent out all of the data
         return TRUE;
      }
   }

   // Something went wrong, return false.

   return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS232Interface::ReadResponse
//
// Description:   Reads from the IO port until it reaches the end of the data
//                or runs out of buffer size
/////////////////////////////////////////////////////////////////////////////
int CRS232Interface::ReadResponse(CString &strBuff)
{
   ULONG numin;
   char recv;

   strBuff = "";

   if (IsValidComm()) {
      // Read from the port until we hit the end of the data or the ReadFile  operation fails
      while (ReadFile(m_hCom, &recv, 1, &numin, NULL) && recv != '\r' && numin != 0) 
		{
         // Throw away plain linefeeds
         if (recv != '\n') {
            // Add a character to the buffer (We're using \r as our terminating character).
            strBuff += recv;
         }
      }
   }

   return strBuff.GetLength();
}

BOOL CRS232Interface::IsValidComm()
{
   return (m_hCom != INVALID_HANDLE_VALUE);
}

void CRS232Interface::InvalidateComm()
{
   // Close the file handle
   if (IsValidComm()) {
      CloseHandle(m_hCom);
      m_hCom = INVALID_HANDLE_VALUE;
   }
}
