// RS485Interface.cpp: implementation of the CRS485Interface class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CommTest.h"
#include "RS485Interface.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS485Interface::CRS485Interface
//
// Description:   Opens the communications channel with the I/O module.  
//                Only one version of this object can be instantiated
//                throughout the system, so there is a critical section lock
//                at the beginning of the constructor, and an unlock in the
//                destructor
/////////////////////////////////////////////////////////////////////////////
CRS485Interface::CRS485Interface(int Address, CString ComPort, int BaudRate)
{
   // Lock the serial object so only one can be used at a time (this is now a multiple-thread
   // safe object)
   m_CommCritSect.Lock();

   DCB dcb;
   BOOL fSuccess;

   if (TRUE) { // Will eventually have conditions for whether or not we do this, I think...
      // Must be OPEN_EXISTING... strPort should be "COM1", "COM2", etc...
      m_iAddr = Address;

      CString strPort = ComPort;
      m_hCom = CreateFile(strPort.GetBuffer(strPort.GetLength()+1), GENERIC_READ | GENERIC_WRITE, 0,
                          NULL, OPEN_EXISTING, 0, NULL);

      if (m_hCom == INVALID_HANDLE_VALUE) {
          // Handle error
         TRACE("CRS485Interface::CRS485Interface -- Invalid Comm Handle\n");
         return;
      }

      fSuccess = GetCommState(m_hCom, &dcb);

      if (!fSuccess) {
          // Handle the error
         TRACE("CRS485Interface::CRS485Interface -- GetCommState() error\n");
      }

      // Fill in the DCB: baud=whatever is set, 8 data bits, no parity, 1 stop bit
      dcb.BaudRate = BaudRate;
      dcb.ByteSize = 8;
      dcb.Parity   = NOPARITY;
      dcb.StopBits = ONESTOPBIT;

      // I'm sure stuff could be different here... but this works so don't touch it :P
      dcb.fOutxCtsFlow = FALSE;
      dcb.fOutxDsrFlow = FALSE;
      dcb.fDtrControl = DTR_CONTROL_DISABLE;
      dcb.fRtsControl = RTS_CONTROL_DISABLE;
      dcb.fDsrSensitivity = FALSE;
      dcb.fTXContinueOnXoff = TRUE;
      dcb.fOutX = FALSE; // **IMPORTANT**
      dcb.fInX = FALSE; // **IMPORTANT**
      dcb.fAbortOnError = FALSE;
      dcb.EvtChar = '\0';

      fSuccess = SetCommState(m_hCom, &dcb);

      if (!fSuccess) {
          // Handle the error.
         TRACE("CRS485Interface::CRS485Interface -- SetCommState() error\n");
      }

      // Important... set our timeout values
      COMMTIMEOUTS cto;

      cto.ReadIntervalTimeout         = 20 * (19200/BaudRate);
      cto.ReadTotalTimeoutMultiplier  = 0;
      cto.ReadTotalTimeoutConstant    = 750;
      cto.WriteTotalTimeoutMultiplier = 0;
      cto.WriteTotalTimeoutConstant   = 750;
      fSuccess = SetCommTimeouts(m_hCom, &cto);

      if (!fSuccess) {
         // Handle the error.
         TRACE("CRS485Interface::CRS485Interface -- SetCommTimeouts() error\n");
      }
   }
   else {
      // We're in demo mode... make the handle invalid so that none of the 
      // functions get used.
      m_hCom = INVALID_HANDLE_VALUE;
   }

   m_iInterfaceType = MODE_RS485;
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS485Interface::~CRS485Interface
//
// Description:   Destructor... closes the handle to the com device and unlocks
//                the critical section.
/////////////////////////////////////////////////////////////////////////////
CRS485Interface::~CRS485Interface()
{
   // Close the file handle
   if (IsValidComm()) {
      CloseHandle(m_hCom);
   }

   // Make sure we unlock the serial object!
   m_CommCritSect.Unlock();
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS485Interface::SendCommand
//
// Description:   Sends and validates a command string to the instrument
/////////////////////////////////////////////////////////////////////////////
BOOL CRS485Interface::SendCommand(CString strCmd)
{
   ULONG numout, numin;
   UCHAR inbuff[256];

   if (IsValidComm()) 
   {
      // Prepend the correct address onto the string
      
      strCmd = "f#a" + strCmd;
      strCmd.SetAt(0, UCHAR(0xFF));
      strCmd.SetAt(2, m_iAddr);

      AfxGetApp()->SetThreadPriority(THREAD_PRIORITY_TIME_CRITICAL);

      // NOTE: As a hardware precaution, we always set the one that needs to go 
      // high first.  This probably only applies to using off the shelf 232 to 485 converters
      EscapeCommFunction(m_hCom, SETRTS);
      EscapeCommFunction(m_hCom, CLRDTR);
      Sleep(2);

      TRACE("SEND: %s\n", strCmd);

      // Now write the buffer out to the com port, with the appropriate control lines set above.
      WriteFile(m_hCom, strCmd.GetBuffer(0), strCmd.GetLength(), &numout, NULL);

      // This clears the echoed characters from our input buffer
      ReadFile(m_hCom, inbuff, strCmd.GetLength(), &numin, NULL);

      // NOTE: As a hardware precaution, we always set the one that needs to go 
      // high first.  This probably only applies to using off the shelf 232 to 485 converters
      EscapeCommFunction(m_hCom, SETDTR);
      EscapeCommFunction(m_hCom, CLRRTS);

      AfxGetApp()->SetThreadPriority(THREAD_PRIORITY_NORMAL);

      if (numout == (ULONG)strCmd.GetLength() && numout == numin) {
         // Ok, we sent out all of the data
         return TRUE;
      }
   }

   // If we're here, something went wrong, return false.
   return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CRS485Interface::ReadResponse
//
// Description:   Reads from the IO port until it reaches the end of the data
//                or runs out of buffer size
/////////////////////////////////////////////////////////////////////////////
int CRS485Interface::ReadResponse(CString &strBuff)
{
   ULONG numin, addr = 0xFFFF;
   UCHAR recv;

   // So we don't get any garbage data left over
   recv = '\0';
   strBuff = "";

   if (IsValidComm()) {

      // First burn through the input until we get a '#', signalling a start
      while (ReadFile(m_hCom, &recv, 1, &numin, NULL) && numin == 1 && recv != '#')
      {
         TRACE("recv: %c:%x\n", recv,recv);  
      }

      if (recv == '#') {
         // Read in the address
         if (ReadFile(m_hCom, &recv, 1, &numin, NULL) && numin == 1) {
            // We successfully got it, we can continue on.
            addr = recv;

            TRACE("addr: %ld\n", addr);

            // Read from the port until we hit the end of the data or the ReadFile operation fails
            while (ReadFile(m_hCom, &recv, 1, &numin, NULL) && numin == 1 && recv != '\r') {
               // Throw away plain linefeeds and 0xFF chars (which we get a lot of)
               if (recv != '\n' && recv != 0xFF) {
                  // Add a character to the buffer (We're using \r as our terminating character).
                  strBuff += recv;
               
               }
            }
         }
      }
   }

   if (addr != m_iAddr) {
      // The address is incorrect... oops.  Pretend this never happened.
      strBuff = "";
   }

   TRACE("strBuff: %s\n", strBuff);
   TRACE("LENGTH: %d\n", strBuff.GetLength());
   for(int i = 0; i < strBuff.GetLength(); i++)
   {
      TRACE("strBuff[%d]: %d\n", i, strBuff.GetAt(i));
   }

   return strBuff.GetLength();
}

BOOL CRS485Interface::IsValidComm()
{
   return (m_hCom != INVALID_HANDLE_VALUE);
}

void CRS485Interface::InvalidateComm()
{
   // Close the file handle
   if (IsValidComm()) {
      CloseHandle(m_hCom);
      m_hCom = INVALID_HANDLE_VALUE;

      TRACE("Addr %d\n", m_iAddr);
   }
}
